/***************************************************************************
 *
 * File name               :browser.c
 * 
 * Description             :Conference manager integrated with CIV project
 *
 * Authors                 :Janusz Bulawa & Tomasz Stachowiak, NPAC
 *
 * Date                    :09-27-96
 *
 ***************************************************************************/

#define XP_UNIX

#include <Xm/Xm.h>
#include "launch.c"
#include "npapi.h"
#include <stdio.h>

#include <Xm/PushB.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Form.h>
#include <Xm/DialogS.h>
#include <X11/Shell.h>
#include <Xm/MessageB.h>

#define MIME_EXT_DESCRIPTION "application/x-dve:.dve"
/* In case of stupid server "text/plain:.dve" */

typedef struct _PluginInstance
{
    Widget	netscape_widget; /* The widget provided by netscape */
    Display    *display;         /* Netscape's display connection */
    uint16	mode;            /* NP_EMBED, NP_FULL, or NP_BACKGROUND */
    uint32 width, height;
    
    int len;
    XtIntervalId timerId;
    XtAppContext appContext;
} PluginInstance;



XtAppContext    AppContext;
Widget          parent;
Widget          bInvite, bReconnect, bPluginInfo, bPluginStart, bPluginStop;
Widget          bAccept, bReject, bLeave, bReturn, bAddM;
Widget          messagelabel;
Widget          pluginsLabel, membersLabel, pluginsList, membersAccList;
XmStringTable   plugTlist;
XtAppContext	appMem;

/* Add member dialog widgets */
Widget	 mainWin, mainForm, textLine, bMemOK, bMemCancel,
         usr_text, host_text;


/* my indicator of existing conference */
int conferenceExist = 0;

Arg wArg[10];
int numArg;
XmString label;
int err;
int create_recipient_list;

int connectDVE();
static void manage(void);
static void invite(void);
static void reconnect(void);
static void addmem(void);
static void pluginInfo(Widget w, XtPointer data1, XtPointer data2);
static void pluginStart(Widget w, XtPointer data1, XtPointer data2);
static void pluginStop(Widget w, XtPointer data1, XtPointer data2);
static void acc(void);
static void reject(void);
static void leave(void);
static void retrn(void);
static void addMemOk();
static void addMemCancel();
static void partyAccepted(DveSession dveSession, DveEvent* event);
static void partyRejected(DveSession dveSession, DveEvent* event);
static void partyLeft(DveSession dveSession, DveEvent* event);
static void partyNew(DveSession dveSession, DveEvent* event);
static void changeDND(DveSession dveSession, DveEvent* event);
static void SetMessage(const char *msg);

int introduceYourself(DveSession *session,
		      DveSession *privatesession);
int init_plugins(DveSession session);
void     PluginReady(pluginObject plugin);
void     PluginDone(pluginObject plugin);
int init_members(DveSession session);


extern DveError dveError;

/***********************************************************************
 *
 * Implementations of Netscape's plugin API functions
 *
 ***********************************************************************/

char *
NPP_GetMIMEDescription(void)
{
#ifdef DISP
  printf( "OpenDVE plugin:MIME description loaded\n");  
#endif
  return( MIME_EXT_DESCRIPTION );
}

NPError
NPP_Initialize(void)
{
#ifdef DISP
  printf( "OpenDVE:Plugin Initialized\n");  
#endif
  return NPERR_NO_ERROR;
}

void
NPP_Shutdown(void)
{
#ifdef DISP
  printf( "OpenDVE:Plugin Shutdown called\n");  
#endif
}


NPError 
NPP_New(NPMIMEType pluginType,
	NPP instance,
	uint16 mode,
	int16 argc,
	char* argn[],
	char* argv[],
	NPSavedData* saved)
{
        PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
		
	instance->pdata = NPN_MemAlloc(sizeof(PluginInstance));
	
	This = (PluginInstance*) instance->pdata;
#ifdef DISP
	  printf( "OpenDVE: NPP_New called\n");
#endif
	if (This != NULL)
	{
   	        int i;
		This->netscape_widget = NULL;
		This->display = NULL;
		This->mode = NP_FULL;   
		This->width = 400;
		This->height = 500;
#ifdef DISP
	  printf( "OpenDVE: NPP_New called Mode=%d\n", This->mode);  
#endif

		/*--- Start custom code */
		
		information = NULL;
		create_recipient_list=0;
		appData.numInv=0;
		appData.numAcc=0;
		appData.numRej=0;
		appData.memDveAccList=NULL;
		appData.initPath=NULL;
		appData.helpPath=NULL;
		mbrs=NULL;

		This->len = 0;
		This->timerId = 0;

		/*--- End custom code */
		
		return NPERR_NO_ERROR;
	}
	else
		return NPERR_OUT_OF_MEMORY_ERROR;
}/* NPP_New */


NPError 
NPP_Destroy(NPP instance, NPSavedData** save)
{
	PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;

#ifdef DISP
	printf( "OpenDVE: NPP_Destroy called\n");  
#endif
        dveConfMgrLeaveConference(appData.session);
	free(mbrs);
	free(localURL);
	dvePluginMgrClose();
	QuitPlugin(appData.session);
	QuitPlugin(appData.privateSession);

	if (This != NULL) {

		/*--- Start custom code */

		/* Stream is closed, we have the data, so start a timer to
		 * call the draw routine
		 */

		if(This->timerId)
		    XtRemoveTimeOut(This->timerId);
		This->timerId = 0;

		/*--- End custom code */

		NPN_MemFree(instance->pdata);
		instance->pdata = NULL;
	}

	return NPERR_NO_ERROR;
}/* NPP_Destroy */

NPError 
NPP_SetWindow(NPP instance, NPWindow* window)
{
	PluginInstance* This;

	Pixel bg, fg;
	int num_fonts;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;
#ifdef DISP	
	printf( "OpenDVE: NPP_SetWindow called for %d\n", (int)instance);  
#endif

	This->display = ((NPSetWindowCallbackStruct *)window->ws_info)->display;
	This->netscape_widget = XtWindowToWidget(This->display, (Window)window->window);
	This->width = window->width;
	This->height = window->height;
	
	This->appContext=XtWidgetToApplicationContext(This->netscape_widget);
	AppContext = This->appContext;
	parent=This->netscape_widget;
	
	label = XmStringCreateSimple("Invite");
	bInvite = XtVaCreateManagedWidget("invite", xmPushButtonWidgetClass,
					 parent,
					 XmNlabelString, label, 
					 XmNx, 10, 
					 XmNy, 10,
					 XmNwidth, 65,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bInvite, XmNactivateCallback, invite, NULL);
	XtSetSensitive(bInvite, False);
	
	label = XmStringCreateSimple("Reconnect");
	bReconnect = XtVaCreateManagedWidget("reconnect", xmPushButtonWidgetClass,
					 parent,
					 XmNlabelString, label, 
					 XmNx, 80, 
					 XmNy, 10,
					 XmNwidth, 75,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bReconnect, XmNactivateCallback, reconnect, NULL);
	XtSetSensitive(bReconnect, False);
	
	label = XmStringCreateSimple("PluginInfo");
	bPluginInfo = XtVaCreateManagedWidget("plugins",
					      xmPushButtonWidgetClass,
					      parent,
					 XmNlabelString, label, 
					 XmNx, 160, 
					 XmNy, 10,
					 XmNwidth, 85,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bPluginInfo, XmNactivateCallback, pluginInfo, NULL);
	XtSetSensitive(bPluginInfo, False);

	label = XmStringCreateSimple("PluginStart");
	bPluginStart = XtVaCreateManagedWidget("pluginStart",
					      xmPushButtonWidgetClass,
					      parent,
					 XmNlabelString, label, 
					 XmNx, 250, 
					 XmNy, 10,
					 XmNwidth, 85,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bPluginStart, XmNactivateCallback, pluginStart, NULL);
	XtSetSensitive(bPluginStart, False);

	label = XmStringCreateSimple("PluginStop");
	bPluginStop = XtVaCreateManagedWidget("pluginStop",
					      xmPushButtonWidgetClass,
					      parent,
					 XmNlabelString, label, 
					 XmNx, 340, 
					 XmNy, 10,
					 XmNwidth, 85,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bPluginStop, XmNactivateCallback, pluginStop, NULL);
	XtSetSensitive(bPluginStop, False);

	label = XmStringCreateSimple("Accept");
	bAccept = XtVaCreateManagedWidget("accept", xmPushButtonWidgetClass,
					 parent,
					 XmNlabelString, label, 
					 XmNx, 10, 
					 XmNy, 50,
					 XmNwidth, 65,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bAccept, XmNactivateCallback, acc, NULL);
	XtSetSensitive(bAccept, False);

	label = XmStringCreateSimple("Reject");
	bReject = XtVaCreateManagedWidget("reject", xmPushButtonWidgetClass,
					 parent,
					 XmNlabelString, label, 
					 XmNx, 80, 
					 XmNy, 50,
					 XmNwidth, 65,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bReject, XmNactivateCallback, reject, NULL);
	XtSetSensitive(bReject, False);
 
	label = XmStringCreateSimple("Leave");
	bLeave = XtVaCreateManagedWidget("leave", xmPushButtonWidgetClass,
					 parent,
					 XmNlabelString, label, 
					 XmNx, 150, 
					 XmNy, 50,
					 XmNwidth, 65,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bLeave, XmNactivateCallback, leave, NULL);
	XtSetSensitive(bLeave, False);

	label = XmStringCreateSimple("Return");
	bReturn = XtVaCreateManagedWidget("return", xmPushButtonWidgetClass,
					 parent,
					 XmNlabelString, label, 
					 XmNx, 220, 
					 XmNy, 50,
					 XmNwidth, 65,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bReturn, XmNactivateCallback, retrn, NULL);
	XtSetSensitive(bReturn, False);

	label = XmStringCreateSimple("Add Mem");
	bAddM = XtVaCreateManagedWidget("addMem", xmPushButtonWidgetClass,
					 parent,
					 XmNlabelString, label, 
					 XmNx, 290, 
					 XmNy, 50,
					 XmNwidth, 65,
					 XmNheight, 30,
					 NULL);

	XmStringFree(label);
	XtAddCallback(bAddM, XmNactivateCallback, addmem, NULL);
	XtSetSensitive(bAddM, True);

	messagelabel = XtVaCreateManagedWidget( "messLab",
			xmLabelWidgetClass,
			parent,
			XmNx, 20,
			XmNy, 90,
			XmNwidth, (This->width)-20,
			XmNheight, 30,
			NULL );
	SetMessage("Conference preparation... Wait...");

	XtRealizeWidget(parent);

	return NPERR_NO_ERROR;

} /* NPP_SetWindow */


NPError 
NPP_NewStream(NPP instance,
	      NPMIMEType type,
	      NPStream *stream, 
	      NPBool seekable,
	      uint16 *stype)
{
	PluginInstance* This;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	This = (PluginInstance*) instance->pdata;

/*--- Start custom code */
#ifdef DISP
	printf( "OpenDVE: NPP_NewStream called\n");  
#endif

	*stype = NP_NORMAL;
	
	localURL = changeURL(stream->url);
	
/*--- End custom code */
	return NPERR_NO_ERROR;
}/* NPP_NewStream */


int32 
NPP_WriteReady(NPP instance, NPStream *stream)
{
  
	PluginInstance* This;
	if (instance != NULL)
		This = (PluginInstance*) instance->pdata;
	
#ifdef DISP
	printf( "OpenDVE: NPP_WriteReady returns %d\n", req);  
#endif

	return 1024; 
	
}


int32 
NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
	int i;
	PluginInstance* This;
	

	if (instance != NULL)
	{
		This = (PluginInstance*) instance->pdata;
	      }

	information = XtRealloc(information, This->len + len + 2);

	strncpy(&(information[This->len]), buffer, len);

	This->len += len;

	information[This->len] = '\0';
	
	return len;		/* The number of bytes accepted */
}


NPError 
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
	PluginInstance* This;
	XCharStruct bounds;
	int direction, ascent, descent;

	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
	This = (PluginInstance*) instance->pdata;
	

	/*
	 * adding timer for the main initialization procedure 
	 */
	This->timerId = XtAppAddTimeOut(This->appContext,
					100,
					start_conference,
					(XtPointer)instance); 
	return NPERR_NO_ERROR;
}


void 
NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{
}

void 
NPP_Print(NPP instance, NPPrint* printInfo)
{
}




/*************************************************************************
  *
  *  Callbacks invoked by buttons
  *
  ************************************************************************/
showMessage(Widget w, char *msg)
{
  Widget dialog;
  XmString text;

  text = XmStringCreateSimple(msg);
  dialog = XmCreateMessageDialog(w, "message", NULL, 0);
  XtVaSetValues(dialog,
		XmNmessageString, text,
		XmNdialogType, XmDIALOG_INFORMATION,
		/*		XmNdialogTitle, "Message",*/
		NULL);
  XmStringFree(text);

  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
  XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));

  XtManageChild(dialog);
  XtPopup(XtParent(dialog), XtGrabNone);
}

int connectDVE()
{
    if ((err=introduceYourself(&(appData.session),
			       &(appData.privateSession)))<0)
    {
        SetMessage("Can not connect to Conference Engine... Check and try again...");
        initLocalOpenDVE();
	return err;   
    }   
    SetMessage("Connected to Conference Engine...");
    fflush(stdout);
    SetMessage("Initializing conferencing panel...");
    if ((err=init_plugins(appData.session))<0)
    { 
        SetMessage("Can not initialize conferencing panel(plugins)... Aborting...");
        return err;
    }
    SetMessage("Conferencing panel ready to use.");
    appData.initiatorHandle=appData.localHandle;

    return 0;
}

static void
reconnect(void)
{
  if ((err=connectDVE())<0)
    return;
  XtSetSensitive(bReconnect, False);
  return;
}


static void
invite(void)
{
  SetMessage("Inviting members... Please, be patient...");
   
  err = dveConfMgrInitiateConference(appData.session, mbrs, appData.numInv);
  if (err<0)
    {
      printf("Invitation error!!!: %s\n", dveErrorString(dveError));
      return;
    }
    
  XtSetSensitive(bInvite, False);
  XtSetSensitive(bAddM, True);
    
    if ((err=init_members(appData.session))<0)
    {
        SetMessage("Can not initialize conferencing panel(members)... Aborting...");
        return;
    }
    
  return;
}

/***********************************
 * 
 * Add member dialog
 * 
 **********************************/
void
addmem()
{
  Widget usr_lab, host_lab;

  mainWin = XtVaCreatePopupShell( "mainWin",
				 xmDialogShellWidgetClass,
				 parent,
				 XmNx, 370,
				 XmNy, 300,
				 XmNwidth, 500,
				 XmNheight, 100,
				 XmNtitle, "Adding a member",
				 XmNiconName, "",
				 XmNmwmDecorations, 2|16|32,
				 XmNmwmFunctions, 4|8|32,
				 NULL );

  mainForm = XtVaCreateManagedWidget( "mainForm",
			xmFormWidgetClass,
			mainWin,
			XmNresizePolicy, XmRESIZE_NONE,
			XmNunitType, XmPIXELS,
			XmNwidth, 500,
			XmNheight, 100,
			NULL );


  usr_lab = XtVaCreateManagedWidget("User:", xmLabelGadgetClass, mainForm,
				    XmNx, 10, 
				    XmNy, 10,
				    NULL);

  usr_text = XtVaCreateManagedWidget("usr_text", xmTextWidgetClass, mainForm,
				     XmNcolumns, 10,
				     XmNx, 70, 
				     XmNy, 10,
				     NULL);


  host_lab = XtVaCreateManagedWidget("Host:", xmLabelGadgetClass, mainForm,
				    XmNx, 200, 
				    XmNy, 10,
				    NULL);

  host_text = XtVaCreateManagedWidget("host_text", xmTextWidgetClass, mainForm,
				     XmNcolumns, 20,
				     XmNx, 250, 
				     XmNy, 10,
				     NULL);



  label = XmStringCreateSimple("OK");
  bMemOK = XtVaCreateManagedWidget("ok", xmPushButtonWidgetClass,
				 mainForm,
				 XmNlabelString, label, 
				 XmNx, 170, 
				 XmNy, 70,
				 XmNwidth, 70,
				 XmNheight, 25,
				 NULL);

  XmStringFree(label);
  XtAddCallback(bMemOK, XmNactivateCallback, addMemOk, NULL);
  XtSetSensitive(bMemOK, True);

  label = XmStringCreateSimple("Cancel");
  bMemCancel = XtVaCreateManagedWidget("cancel", xmPushButtonWidgetClass,
				 mainForm,
				 XmNlabelString, label, 
				 XmNx, 300, 
				 XmNy, 70,
				 XmNwidth, 70,
				 XmNheight, 25,
				 NULL);

  XmStringFree(label);
  XtAddCallback(bMemCancel, XmNactivateCallback, addMemCancel, NULL);
  XtSetSensitive(bMemCancel, True);

  return;
}

/*
 * Add member dialog callbacks 
 */

/* OK callback */
static void
addMemOk(w, cdata, cbs)
Widget w;
XtPointer cdata;
XmPushButtonCallbackStruct *cbs;
{
  DveConfMgrUserAddr addr;

  addr.userName = XmTextGetString(usr_text);
  addr.hostName = XmTextGetString(host_text);
  addr.addrs = NULL;
  appData.numInv = 1;
  if(conferenceExist)
    {
      dveConfMgrAddMember(appData.session, &addr);
    }
  else
    {
      dveConfMgrInitiateConference(appData.session, &addr, 1);
      conferenceExist = True;
    }
  XtDestroyWidget(XtParent(XtParent(w)));
}

/* Cancel callback */
static void
addMemCancel(w, cdata, cbs)
Widget w;
XtPointer cdata;
XmPushButtonCallbackStruct *cbs;
{
  XtDestroyWidget(XtParent(XtParent(w)));
}

static void
pluginInfo(Widget w, XtPointer data1, XtPointer data2)
{
  if (appData.currPlug >= 0)
  {
    printf("Name    : %s\n", dvePluginList[appData.currPlug]->name);
    printf("Title   : %s\n", dvePluginList[appData.currPlug]->title);
    printf("Port    : %d\n", dvePluginList[appData.currPlug]->port);
    printf("Proc ID : %d\n", dvePluginList[appData.currPlug]->processID);
    printf("Path    : %s\n", dvePluginList[appData.currPlug]->basePath);
    fflush(stdout);
  }
  
  return;
}

static void
pluginStart(Widget w, XtPointer data1, XtPointer data2)
{
  static JBPluginManagementArg dane;

  dane.port=dvePluginList[appData.currPlug]->port;
  SendPluginMessageGeneric(appData.session, JB_CONF_MANAGER_PRIVATE,
			   JB_CONF_MANAGER_PRIVATE,
			   dvePluginManagementMsgTagShow,
			   -1, &dane, sizeof(dane));
  
  if ((dvePluginList[appData.currPlug]->processID) <=0)
  {
    dvePluginMgrStartPlugin(appData.session,
	   		    dvePluginList[appData.currPlug],
			    PLUGIN_MAPPED);
  }
  else
    {
      printf("Selected plugin is already active!");
      fflush(stdout);
    }
  
  return;
}

static void
pluginStop(Widget w, XtPointer data1, XtPointer data2)
{
  static JBPluginManagementArg dane;

  dane.port=dvePluginList[appData.currPlug]->port;
  SendPluginMessageGeneric(appData.session, JB_CONF_MANAGER_PRIVATE,
			   JB_CONF_MANAGER_PRIVATE,
			   dvePluginManagementMsgTagQuit,
			   -1, &dane, sizeof(dane));
  
  if ((dvePluginList[appData.currPlug]->processID) >0)
  {
    dvePluginMgrQuitPlugin(appData.session,
	   		    dvePluginList[appData.currPlug]);
  }
  else
    {
      printf("Selected plugin is not running!");
      fflush(stdout);
    }
  
  return;
}

void
accListCallback(DveMotifRecipientList list,
		DveMotifRecipientListCallbackStruct* data)
{
  return;
}

static void
acc(void)
{
  dveConfMgrAcceptInvitation(appData.session);
  SetMessage("Invitation to teleconference has been accepted.\n");
  XtSetSensitive(bAccept, False);
  XtSetSensitive(bReject, False);
  XtSetSensitive(bLeave, True);
  create_recipient_list=1;
    
  return;
}

static void
reject(void)
{
  dveConfMgrRejectInvitation(appData.session);
  SetMessage("Invitation to teleconference has been rejected!\n");
  XtSetSensitive(bAccept, False);
  XtSetSensitive(bReject, False);
  exit(0);
  return;
}

static void
leave(void)
{
  if (dveLeaveOffice(appData.session)<0)
      printf("Error when leaving Office: %s\n", dveErrorString(dveError));
  SetMessage("Local user left office!\n");
  XtSetSensitive(bLeave, False);
  XtSetSensitive(bReturn, True);
/*  fprintf(stdout,"left\n");
  fflush(stdout);*/
  return;
}

static void
retrn(void)
{
  if (dveReturnToOffice(appData.session)<0)
      printf("Error when returning to Office: %s\n", dveErrorString(dveError));
  SetMessage("Local user returned to office!\n");
  XtSetSensitive(bLeave, True);
  XtSetSensitive(bReturn, False);
  return;
}

static void
partyAccepted(DveSession dveSession, DveEvent* event)
{
  /*  fprintf(stdout,"user: %s on host: %s accepted invitation\n", event->arg.memberAccepted.member.userName, event->arg.memberAccepted.member.hostName);*/
    
  if (appData.memDveAccList)
    dveMotifRecipientListHandleDveEvent(appData.memDveAccList, event);

  appData.numAcc+=1;

  if ((appData.numAcc + appData.numRej)==appData.numInv)
    {
      if ((err=init_members(appData.session))<0)
	{
	  SetMessage("Can not initialize conferencing panel(members)... Aborting...");
	  return;
	}

      DvePartListClrAll(&(appData.memAccList));
      appData.memDveAccList = dveMotifRecipientListCreate(appData.session,
							 membersAccList,
							 &(appData.memAccList),
							 False);
      dveMotifRecipientListSetCallback(appData.memDveAccList, accListCallback);
    }
  fflush(stdout);
  return;
}

static void   
partyRejected(DveSession dveSession, DveEvent* event)
{
  char text[100];

  sprintf(text, "User %s@%s\nrejected invitation!\n", 
	  event->arg.memberRejected.member.userName, 
	  event->arg.memberRejected.member.hostName);
  showMessage(parent, text);

  /*  fprintf(stdout,"user: %s on host: %s rejected invitation\n", 
	  event->arg.memberRejected.member.userName, 
	  event->arg.memberRejected.member.hostName);
  fflush(stdout);
  */
  /*
  appData.numRej+=1;
  if (appData.memDveAccList)
    dveMotifRecipientListHandleDveEvent(appData.memDveAccList, event);
  if ((appData.numAcc + appData.numRej)==appData.numInv)
  {
    DvePartListClrAll(&(appData.memAccList));
    appData.memDveAccList = dveMotifRecipientListCreate(appData.session,
							membersAccList,
							&(appData.memAccList),
							False);
    dveMotifRecipientListSetCallback(appData.memDveAccList, accListCallback);
  }
  fflush(stdout);
  */
  return;
}

static void
partyLeft(DveSession dveSession, DveEvent* event)
{
/*  fprintf(stdout,"user: %s on host: %s left the conference\n", event->arg.memberLeft.member.userName, event->arg.memberLeft.member.hostName);
  fflush(stdout);*/
  if (appData.memDveAccList)
    dveMotifRecipientListHandleDveEvent(appData.memDveAccList, event);
  return;
}

static void
partyNew(DveSession dveSession, DveEvent* event)
{
  fprintf(stdout,"user: %s on host: %s\nhas been invited to join teleconference\n", event->arg.newMember.member.userName, event->arg.newMember.member.hostName);
  if (appData.memDveAccList)
    dveMotifRecipientListHandleDveEvent(appData.memDveAccList, event);
  fflush(stdout);
  return;
}

static void
changedDND(DveSession dveSession, DveEvent* event)
{
  fprintf(stdout,"Local DND flag is set to %s\n", (event->arg.DNDChanged.DND)?"True":"False");
  fflush(stdout);
  return;
}

static void
SetMessage(const char *msg)
{
  XmString str=XmStringCreateSimple((char*)msg);
  XtVaSetValues(messagelabel,
		XmNlabelString, str,
		NULL);
  XmStringFree(str);
  return;
}

/***************************************************************************

  Definitions of OpenDVE-specific functions

***************************************************************************/  

int initLocalOpenDVE()
{
  
  if ((err=check_process("ISregistrar", ""))<0)
  {
    failure("ISregistrar problem\n");
    return err;
  }
  
  if ((err=check_process("ISconferenceEngine", ""))<0)
  {
    failure("ISconferenceEngine problem\n");
    return err;
  }
 
  return 0;
}


void exit_conference(void)
{
  dveConfMgrLeaveConference(appData.session);
  QuitPlugin(appData.session);
}

DveConfMgrInvitationCallback invited(DveSession dveSession,
				     DveConfMgrInvitedArg* arg)
{
  int res;

  /*printf("RECEIVED INVITATION! from: %s at %s\n", (arg->parts[arg->initiatorIndex]).userName, (arg->parts[arg->initiatorIndex]).hostName);*/
  appData.numInv = arg->numParts;
/*  printf("Ilosc uczestnikow: %d\n", appData.numInv);*/
  appData.initiatorHandle=(arg->parts[arg->initiatorIndex]).userHandle;

    if ((err=init_members(appData.session))<0)
    {
        SetMessage("Can not initialize members... Aborting...");
        return;
    }

  dveConfMgrAcceptInvitation(appData.session);
  SetMessage("Invitation to teleconference has been accepted.\n");
  XtSetSensitive(bAccept, False);
  XtSetSensitive(bReject, False);
  XtSetSensitive(bLeave, True);
  create_recipient_list=1;
  /*
  XtSetSensitive(bAccept, True);
  XtSetSensitive(bReject, True);
  SetMessage("Press 'ACCEPT' to join the conference or 'REJECT'");*/
  return;
}

void start_conference(void)
{ 

  int err;
 
  XtSetSensitive(bInvite, False);

  /* starting local ISregistrar and ISconferenceEngine if necessary */
  SetMessage("Initializing Conference Engine...");
  if ((err=initLocalOpenDVE())<0)
    {
      SetMessage("Can not initialize OpenDVE... Aborted...");
      return;
    }

  /* connect to DVE */
  if ((err=connectDVE())<0)
    {
      XtSetSensitive(bReconnect, True);
    }
   
}

void started(DveSession dveSession, DveEvent* event)
{
  int ilosc, i;

  ilosc=event->arg.starting.numMembers;
  appData.state=event->arg.starting.state;
  /*  fprintf(stdout, "I am invited! %d %d\n", ilosc, (int)appData.state);
      for (i=0;i<ilosc;i++)
    fprintf(stdout, "%s@%s\n", event->arg.starting.members[i].part.userName, event->arg.starting.members[i].part.hostName);
*/
  if (create_recipient_list)
  {
    DvePartListClrAll(&(appData.memAccList));
    appData.memDveAccList = dveMotifRecipientListCreate(appData.session,
							membersAccList,
							&(appData.memAccList),
							False);
    dveMotifRecipientListSetCallback(appData.memDveAccList, accListCallback);
    create_recipient_list=0;
  }
  fflush(stdout);
  return;
}


QuitPlugin(DveSession dveSession)
{
 
   dveUnregisterWithXt(dveSession);
   dveCloseSession(dveSession);
   return;
}

/*
 * QuitPlugin if requested.
 */

static void
PluginControlMsg(DveSession dveSession, DveUserHandle sender,
                 DveDataPacket* packet)
{
  static JBPluginManagementArg dane;

  memcpy(&dane, packet->data, sizeof(JBPluginManagementArg));
  switch ((DvePluginManagementMsgTag)packet->tag)
   {
     case dvePluginManagementMsgTagShow:
         dvePluginMgrStartPlugin(appData.session,
	   		    dvePluginMgrLocatePluginByPortNumber(dane.port),
			    PLUGIN_MAPPED);
	 XtSetSensitive(bPluginInfo, True);
	 XtSetSensitive(bPluginStart, True);
	 XtSetSensitive(bPluginStop, True);
         break;
     case dvePluginManagementMsgTagQuit:
         dvePluginMgrQuitPlugin(appData.session,
	   		    dvePluginMgrLocatePluginByPortNumber(dane.port));
         break;
   }
}

void
HidePlugin()
{
/*   XtPopdown(MyShell);*/
}
 
void
ShowPlugin()
{
/*   XtPopup(MyShell, XtGrabNone);*/
}
 
int
introduceYourself(DveSession *session, DveSession *privatesession)
{
   /*
    * Start up OpenDVE.
    */
   if (!((*session)=dveOpenSession(JB_CONF_MANAGER_PORT)))
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      switch (dveError)
	{
	case dveErrOutOfMemory:
	  fprintf(stderr, "Error Condition: [Low memory.]\n");
	  break;
	  
	case dveErrNotFound:
	  fprintf(stderr, 
		  "Error Condition: [Conference Engine not found.]\n");
	  break;
	  
	case dveErrConnRefused:
	  fprintf(stderr, 
		  "Error Condition: [Engine refused connection.]\n");
	  break;
	default:
	  fprintf(stderr, "Unknown Error Condition.\n");
	  break;
	}
       return -1;
   }
   if (!((*privatesession)=dveOpenSession(JB_CONF_MANAGER_PRIVATE)))
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      switch (dveError)
	{
	case dveErrOutOfMemory:
	  fprintf(stderr, "Error Condition: [Low memory.]\n");
	  break;
	  
	case dveErrNotFound:
	  fprintf(stderr, 
		  "Error Condition: [Conference Engine not found.]\n");
	  break;
	  
	case dveErrConnRefused:
	  fprintf(stderr, 
		  "Error Condition: [Engine refused connection.]\n");
	  break;
	default:
	  fprintf(stderr, "Unknown Error Condition.\n");
	  break;
	}
       return -1;
   }

   dveSetUserCallback(*privatesession, JB_CONF_MANAGER_PRIVATE, PluginControlMsg);
   dveSetEventCallback(*session,dveEventTagStarting,started);
   dveSetEventCallback(*session, dveEventTagMemberAccepted, partyAccepted);
   dveSetEventCallback(*session, dveEventTagMemberRejected, partyRejected);
   dveSetEventCallback(*session, dveEventTagMemberLeft, partyLeft);
   dveSetEventCallback(*session, dveEventTagNewMember, partyNew);
   dveSetEventCallback(*session, dveEventTagDNDChanged, changedDND);
   dveConfMgrSetInvitationCallback(*session, invited);
     
   /*
    * Register this event processing with the Xt toolkit.  Without this
    * call OpenDVE events will not be received.
    */
   if (dveRegisterWithXt(*session,AppContext)<0)
   {
      fprintf(stderr, "Unable to register with Xt loop.");
      dveCloseSession(*session);
      return -1;
   }
   if (dveRegisterWithXt(*privatesession,AppContext)<0)
   {
      fprintf(stderr, "Unable to register with Xt loop.");
      dveCloseSession(*session);
      return -1;
   }

   dveReportPresence(*session);
   dveReportPresence(*privatesession);
   appData.localHandle = dveGetLocalUserHandle(*session);
   
   return 0;
}

void
pluginSelected(Widget w, XtPointer data1, XtPointer data2)
{
  XmListCallbackStruct *cbs=(XmListCallbackStruct *)data2;

/*  printf("Selected item: %d\n", cbs->item_position-1);
  printf("Number of items: %d\n", cbs->selected_item_count);
  fflush(stdout);*/
  if (cbs->selected_item_count > 0)
  {
    appData.currPlug = cbs->item_position-1;
    XtSetSensitive(bPluginInfo, True);
    XtSetSensitive(bPluginStart, True);
    XtSetSensitive(bPluginStop, True);
  }
  else
  {
    appData.currPlug = -1;
    XtSetSensitive(bPluginInfo, False);
    XtSetSensitive(bPluginStart, False);
    XtSetSensitive(bPluginStop, False);
  }
  
  return;
}

void
createPluginsInterface()
{
  int i;
  XmString label;

  label=XmStringCreateLocalized("Available plugins:");
  pluginsLabel=XtVaCreateManagedWidget("pluginsLabel",
				       xmLabelGadgetClass,
				       parent,
				       XmNlabelString, label,
				       XmNx, 10,
				       XmNy, 140,
				       XmNwidth, 220,
				       NULL);


  numArg=0;
  XtSetArg(wArg[numArg], XmNx, 10); numArg++;
  XtSetArg(wArg[numArg], XmNy, 170); numArg++;
  pluginsList = XmCreateScrolledList(parent, "Plugins", wArg, numArg);

  plugTlist=NULL;
  XtVaSetValues(pluginsList,
	        XmNitems, plugTlist,
		XmNitemCount, 0,
		XmNvisibleItemCount, 5,
		XmNselectionPolicy, XmSINGLE_SELECT,
		NULL);
					   		
  plugTlist=(XmStringTable)XtMalloc(appData.numPlug*sizeof(XmString *));
  for(i=0; i<appData.numPlug;i++)
  {
    plugTlist[i]=XmStringCreateLocalized(dvePluginList[i]->title);
  }
  
  XtVaSetValues(pluginsList,
                XmNitems, plugTlist,
		XmNitemCount, appData.numPlug,
		XmNvisibleItemCount, 3,
		XmNselectionPolicy, XmSINGLE_SELECT,
		NULL);
  XtAddCallback(pluginsList, XmNsingleSelectionCallback, pluginSelected, NULL);
  XtManageChild(pluginsList);
  
  for(i=0; i<appData.numPlug;i++)
  {
    XmStringFree(plugTlist[i]);
  }
  XtFree(plugTlist);
  
  return;
}

int
init_plugins(DveSession session)
{
  char *home;
  
  appData.initPath=(char*)XtMalloc(256);  
  if (appData.initPath == NULL)
  {
     printf("Can not allocate memory for string! Exiting...\n");
     return -1;
   }

  home = getenv("HOME");
  strcpy(appData.initPath, home);
  strcat(appData.initPath, "/plugins.cfg");
  
  /* HELP VIEWER not used since InSoft's Help library is missing 

     appData.helpPath=(char*)XtMalloc(256);  
  if (appData.helpPath == NULL)
  {
     printf("Can not allocate memory for string! Exiting...\n");
     return -1;
   }
  appData.helpPath="";
  appData.helpPath=getenv("HOME");
  strcat(appData.helpPath, "/DVE/localplugins");
  err=IS_AddHelpPath(session, appData.helpPath);
  printf("Result form addHelpPAth: %d\n", err);
  fflush(stdout);*/

  
  /*  printf("Plugins are searched at: %s\n", appData.initPath);
  fflush(stdout);*/

  err = dvePluginMgrInitialize(session, appData.initPath, PluginReady, PluginDone);

  if (err < 0)
    {
      fprintf(stderr, "Error %d on dvePluginMgrInitialize().\n", 
	      dvePluginMgrErrno);
      return -1;
    }
  XtFree(appData.initPath);
  
  appData.numPlug = dvePluginMgrIdentifyAvailablePlugins(session);
  createPluginsInterface();
  return 0;
}

void
PluginReady(pluginObject plugin)
{
  return;
}

void
PluginDone(pluginObject plugin)
{
  return;
}

void
membersAccSelected(Widget w, XtPointer data1, XtPointer data2)
{
  return;
}

void
membersRejSelected(Widget w, XtPointer data1, XtPointer data2)
{
  return;
}

void
createMembersInterface()
{
  int i;

  /* list of members which accepted invitation */
  label=XmStringCreateLocalized("Active members:");
  membersLabel=XtVaCreateManagedWidget("membersLabel",
				       xmLabelGadgetClass,
				       parent,
				       XmNlabelString, label,
				       XmNx, 240,
				       XmNy, 140,
				       NULL);

  
  numArg=0;
  XtSetArg(wArg[numArg], XmNx, 240); numArg++;
  XtSetArg(wArg[numArg], XmNy, 170); numArg++;
  membersAccList = XmCreateScrolledList(parent, "membersAcc",
					       wArg, numArg);

  XtVaSetValues(membersAccList,
		XmNitemCount, 0/*appData.numInv*/,
		XmNvisibleItemCount, 5,
		XmNselectionPolicy, XmMULTIPLE_SELECT,
		NULL);
					   		
  XtManageChild(membersAccList);
  
}

int
init_members(DveSession session)
{
  createMembersInterface();
  return 0;
}


/*
 * Passes data from this plugin to other conferenced members.
 */
int
SendPluginMessageGeneric(DveSession dveSession, u_int32 protocol, DvePort port,
			u_int32 messageType, int dest, void *data, u_int32 size)
{
   DveDataPacket *packet;
   int returnCode,i;

/*   printf("Size of data to send: %d\n", size);
   fflush(stdout);
  */ 
   packet = (DveDataPacket*) dveMallocDataPacket(dveSession, size);
   packet->tag = (u_int32)messageType;
   packet->pluginProtocol = protocol;
   packet->destPort = port;

/*   printf("SendGeneric to port %d\n", packet->destPort);*/
   DvePartListClrAll(&(packet->destParts));

   if (dest < 0)
   {
      DvePartListSetAll(&(packet->destParts));
   }
   else
   {
      DvePartListSet(dest,&(packet->destParts));
   }

   if (size > 0)
      memcpy(packet->data, data, size);

   returnCode = dveSendDataPacket(dveSession, packet);

   dveFreeDataPacket(packet);

/*   printf("Result from Send(): %d\n", returnCode);*/
   fflush(stdout);

   return(returnCode);
}














